package org.nrg.xnat.repository;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Vector;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethodBase;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.DeleteMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.PutMethod;
import org.apache.commons.io.IOUtils;

import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;

import org.nrg.xnat.desktop.tools.XNATTableParser;
import org.nrg.xnat.desktop.utils.Utils;
import org.nrg.xnat.desktop.utils.XNATThesaurus;
import org.nrg.xnat.fileserver.FileTransfer;

import sun.misc.BASE64Encoder;

public class XNATRestAdapter extends RepositoryManager implements FileTransfer
{
	private final String m_root;
	private final String m_usr;
	private final String m_pass;
	public final static int GET=0, PUT=1, POST=2, DELETE=3;
	
	public boolean SessionInit(Vector params)
	{
		return true;
	}
	public boolean Put(ItemRecord remote,ItemRecord local)
	{
		return false;
	}
	public boolean Get(ItemRecord remote, ItemRecord local)
	{
		String path=remote.GetRelativePath();
		if(path.startsWith("/REST"))
			path=path.substring(5);
		else if(path.startsWith("REST"))
			path=path.substring(4);
		HttpMethodBase get=PerformConnection(GET,path,"");
		if(get==null) return false;
		
		try 
		{
		    final InputStream is = get.getResponseBodyAsStream();
		    final FileOutputStream fos = new FileOutputStream(local.GetAbsolutePath());
		    try 
		    {
		    	IOUtils.copy(is, fos);
		    } finally 
		    {
		    	get.releaseConnection();
		    	fos.close();
		    }
		}catch(Exception e)
		{
			Utils.logger.error("XNATRestAdapter.Get exception",e);
			return false;
		}
		return true;
	}
	public boolean CreateSubject(ItemRecord ir)
	{
		String request="/projects/"+ir.GetTagValue("Project")+"/subjects/"+ir.GetTagValue("Subject");
		final HttpMethodBase method=PerformConnection(PUT,request,"");
		if(method!=null)
		{
			method.releaseConnection();
			return true;
		}
		return false;
	}
	
	public HttpMethodBase PerformConnection(int type, String request, String body)
	{
		final HttpMethodBase method;
		boolean bParams=request.contains("?");
		
		String q=m_root+("/REST/"+request+(bParams?"&":"?")+
				(request.contains("format=")?"":"format=xml")).replace("//","/");
		
		switch(type)
		{
			case GET: 
				method=new GetMethod(q);
//				if(params.length()>0) q+="&"+params;
				break;
			case PUT: method=new PutMethod(q);break;
			case POST: 
				method=new PostMethod(q);
				((PostMethod)method).setRequestBody(body);
				break;
			case DELETE: method=new DeleteMethod(q);break;
			default: return null;
		}
		method.addRequestHeader("Authorization","Basic "+(new BASE64Encoder()).encode(
				(m_usr+":"+m_pass).getBytes()));
		try
		{
			HttpClient client=new HttpClient();
			client.getHttpConnectionManager().getParams().setConnectionTimeout(5000);			
			int status=client.executeMethod(method);
			if(HttpStatus.SC_OK!=status)
			{
				System.err.println("XNAT Rest adapter.PerformConnection: HTTP response "+status);
//				Utils.logger.error("XNAT Rest adapter.PerformConnection: HTTP response "+status);
				return null;
			}
			
//			System.out.println(method.getResponseBodyAsString());
		}catch(Exception ex)
		{
			Utils.logger.error("XNAT Rest adapter: HTTP connection failed", ex);
			System.out.println("REST HTTP connecton failed");
			return null;
		}
		return method;
	}
	
	public boolean VerifyConnection()
	{
		HttpMethodBase m=PerformConnection(GET,"/projects?accessible=true","");
		if(m!=null) m.releaseConnection();
		return (m!=null);
	}

	public XNATRestAdapter(final String server, final String usr, final String pass) 
	{
		m_root=server;
		m_usr=usr;
		m_pass=pass;
	}
	private String FormSubQuery(Collection<ItemTag> context)
	{
		String name,label;
		String result="/";
		for(ItemTag it:context)
		{
			name=it.GetName().toLowerCase()+"s";
			label=it.GetFirstValue();
			result+=name+"/"+label+XNATThesaurus.GetPostfix(it)+"/";
		}
		return result;
	}
	@Override
	public boolean DBItemAdd(ItemRecord item)
	{
		//so far, this is a read-only adapter.
		return false;
	}

	/* (non-Javadoc)
	 * This function returns corresponding files belonging to the lowest defined hierarchical level.
	 */
	@Override
	public ItemRecord[] DBItemFind(ItemRecord template, int maxrecords,
			boolean retrieveMetadata)
	{
		ItemRecord[] records=new ItemRecord[1];
		records[0]=template;
		return records;
	}
	
	/**
	 * Files from given hierarchical level.
	 * @param rel_path      ignored
	 * @param tagsMatching  both name and value for tags in this collection should match
	 * @param tagsDefined   empty array.
	 * @param tagsUndefined ignored
	 * @return
	 */
	public Collection<ItemRecord> DBItemFindEx(String rel_path, Collection<ItemTag> tagsMatching, 
			Collection<String> tagsDefined, Collection<String> tagsUndefined)
	{
		//only resources can have files in them.
		if(((LinkedList<ItemTag>)tagsMatching).getLast().GetName().compareTo("Resource")!=0)
			return new LinkedList<ItemRecord>();
		
		String query=FormSubQuery(tagsMatching);
		LinkedList<TreeMap<String,String>> row_map;
		LinkedList<ItemRecord> res=new LinkedList<ItemRecord>();
		try
		{
			HttpMethodBase get=PerformConnection(GET,query+"/files","");
			if(get==null)
				Utils.logger.error("DBInteFindEx: GET failed");
			row_map=XNATTableParser.GetRows(new SAXReader().read(get.getResponseBodyAsStream()));
			get.releaseConnection();
			ItemRecord ir;
			for(TreeMap<String,String> row:row_map)
			{
				ir=new ItemRecord(null,row.get("URI"));
				ir.TagsSet(tagsMatching);				
				XNATTableParser.SetAllTags(ir, row);
				res.add(ir);
			}
		}
		catch(DocumentException e)
		{
			Utils.logger.error("DBTagValues: Document exception when reading input from "+m_root);
		}
		catch(IOException e)
		{
			Utils.logger.error("DBTagValues: IOException when connecting to "+m_root);
		}
		return res;
	}
	
	/**
	 * Find all child tags for a given hierarchy. 
	 * @param path Contains defined tag hierarchy:
	 * Tag_1: (Name_1=Value_1),...,Tag_N:(Name_N=Value_N)
	 * @param query_tags child tags to be populated
	 * qTag_1: (qName_1=?),...,(qName_M=?)
	 * @return set values in query_tags; map of associated tags for each query tag 
	 */
	public Collection<LinkedList<ItemTag>> DBTagValues (final Collection<ItemTag> path, 
			final Collection<ItemTag> query_tags, final TreeMap<String,Collection<String>> tagMap)
	{
		String query=FormSubQuery(path);
		String q1;
		LinkedList<TreeMap<String,String>> row_map;
		LinkedList<LinkedList<ItemTag>> aTags=new LinkedList<LinkedList<ItemTag>>();
		LinkedList<ItemTag> llit;
		LinkedList<ItemTag> nqt=new LinkedList<ItemTag>();
		try
		{
			ItemRecord ir;
			for(ItemTag it:query_tags)
			{
				q1=query+"/"+it.GetName().toLowerCase()+"s";
				final HttpMethodBase get = PerformConnection(GET,q1+"?"+XNATThesaurus.GetQueryParams(it),"");
				if(get!=null)
				{
					row_map=XNATTableParser.GetRows(new SAXReader().read(get.getResponseBodyAsStream()));
					get.releaseConnection();
					ItemTag qTag=new ItemTag(it.GetName());
					for(TreeMap<String,String> row:row_map)
					{
						String val=row.get(XNATThesaurus.GetIDName(it));
						if(val!=null)
						{
							qTag.AddValue(val);
							ir=new ItemRecord(null,null);
							XNATTableParser.SetAllTags(ir, row);
							llit=new LinkedList<ItemTag>();
							llit.add(new ItemTag(qTag.GetName(), val));
							llit.addAll(ir.GetTagCollection());
							aTags.add(llit);
						}
					}
					nqt.add(qTag);
				}
			}
			query_tags.clear();
			query_tags.addAll(nqt);
		}
		catch(DocumentException e)
		{
			Utils.logger.error("DBTagValues: Document exception when reading input from "+m_root);
		}
		catch(MalformedURLException e)
		{
			Utils.logger.error("DBTagValues: URL exception when trying to connect to "+m_root);
		}
		catch(IOException e)
		{
			Utils.logger.error("DBTagValues: IOException when reading from "+m_root);
		}
		return aTags;
	}

	@Override
	public boolean DBTagAdd(String name)
	{
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean DBTagAttach(ItemRecord item, ItemTag tag)
	{
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean DBTagDelete(String name)
	{
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean DBTagDetach(ItemRecord item, ItemTag tag)
	{
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public String[] DBTagFind(String name)
	{
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean ItemRemove(ItemRecord template)
	{
		// TODO Auto-generated method stub
		return false;
	}

}
